/****************************************************************************
**
** Copyright (C) Datafox GmbH.
** All rights reserved.
** Contact: Datafox GmbH (develop@datafox.de)
**
** This file is part of the DFComDLL module of the Datafox SDK.
**
****************************************************************************/
#pragma once
#ifndef DATAFOX_COMMUNICATION_MODULE_HARDWARE_H
#define DATAFOX_COMMUNICATION_MODULE_HARDWARE_H

// Basisklasse einbinden.
#include "datafox/communication/module_base.h"

// Datenklasse
#include "common/hardware_info.h"
#include <stdlib.h>

/** In diesem Module sind alle Hardwarespezifischen Funktionen enthalten.
*
* Die Aufteilung basiert Hauptsaechlich auf Typen, Komponenten und Optionen.
*
*/
#define IV_MODULE_HARDWARE                       0x0000

// Setzen/Ermitteln des Geraetetyps
#define IV_FUNCTION_SET_TYPE_DEVICE              0x0001
#define IV_FUNCTION_GET_TYPE_DEVICE              0x0002

#define IV_VALUE_TYPE_DEVICE_UNKNOWN                      255 // Unbekannter Gerätetyp
#define IV_VALUE_TYPE_DEVICE_COUNT                         36 // Gesamtzahl der Geraetetypen
#define IV_VALUE_TYPE_DEVICE_ALL                          (~0)// Alle Typen werden unterstuetzt.
#define IV_VALUE_TYPE_DEVICE_PZE                            0 // PZE-MasterIV
#define IV_VALUE_TYPE_DEVICE_ZK                             1 // ZK-MasterIV
#define IV_VALUE_TYPE_DEVICE_BDE                            2 // BDE-MasterIV
#define IV_VALUE_TYPE_DEVICE_MOBIL                          3 // Mobil-MasterIV
#define IV_VALUE_TYPE_DEVICE_MDE                            4 // MDE-BoxIV
#define IV_VALUE_TYPE_DEVICE_AE                             5 // AE-MasterIV
#define IV_VALUE_TYPE_DEVICE_TIMEBOY                        6 // TimeboyIV
#define IV_VALUE_TYPE_DEVICE_EXKLUSIV                       7 // Exklusiv-Line IV
#define IV_VALUE_TYPE_DEVICE_FLEX                           8 // Flex-MasterIV
#define IV_VALUE_TYPE_DEVICE_TIMEBOY_MOBIL_PZE              9 // Timeboy-Mobil-PZE
#define IV_VALUE_TYPE_DEVICE_EVO_2_8_PURE                  10 // EVO 2.8 Pure
#define IV_VALUE_TYPE_DEVICE_EVO_4_3                       11 // EVO 4.3
#define IV_VALUE_TYPE_DEVICE_IPC_EXTENSION_VARIO_5_7       12 // IPC-Extension
#define IV_VALUE_TYPE_DEVICE_IPC_EXTENSION_VARIO_10        13 // IPC-Extension für Vario 10 / Vario 12 / Vario 15
#define IV_VALUE_TYPE_DEVICE_IPC_EXTENSION_Q7              14 // IPC-Extension für Q7
#define IV_VALUE_TYPE_DEVICE_IO_BOX                        15 // IO-Box
#define IV_VALUE_TYPE_DEVICE_KYO_INLOC                     16 // KYO Inloc
#define IV_VALUE_TYPE_DEVICE_MOBIL_BOX                     17 // Mobil-Box
#define IV_VALUE_TYPE_DEVICE_TIMEBOY_TBDOCK                18 // Timeboy-Docking
#define IV_VALUE_TYPE_DEVICE_MOBIL_TBDOCK                  19 // Mobil-Docking
#define IV_VALUE_TYPE_DEVICE_KYO_ONELOC                    20 // KYO Oneloc
#define IV_VALUE_TYPE_DEVICE_EVO_3_5_UNIVERSAL             21 // EVO 3.5 Universal
#define IV_VALUE_TYPE_DEVICE_SAMS_OCL_READER               22 // SAMS-OCL-Reader
#define IV_VALUE_TYPE_DEVICE_EVO_3_5_PURE                  23 // EVO 3.5 Pure
#define IV_VALUE_TYPE_DEVICE_EVO_AGERA                     24 // EVO Agera
#define IV_VALUE_TYPE_DEVICE_KYO_CENLOC                    25 // KYO Cenloc
#define IV_VALUE_TYPE_DEVICE_VARIO_10                      26
#define IV_VALUE_TYPE_DEVICE_VARIO_12                      27
#define IV_VALUE_TYPE_DEVICE_VARIO_15                      28
#define IV_VALUE_TYPE_DEVICE_VARIO_17                      29
#define IV_VALUE_TYPE_DEVICE_IPC_EVO_12_1                  30
#define IV_VALUE_TYPE_DEVICE_IPC_EVO_15                    31
#define IV_VALUE_TYPE_DEVICE_IPC_EVO_18_5                  32
#define IV_VALUE_TYPE_DEVICE_IPC_EVO_24                    33
#define IV_VALUE_TYPE_DEVICE_EVO_INTERA_2                  34
#define IV_VALUE_TYPE_DEVICE_EVO_4_6_FLEX_KEY              35
#define IV_VALUE_TYPE_DEVICE_EVO_5_0_PURE                  36


// Typen in Textform
#define IV_VALUE_TYPE_DEVICE_NAME                          "MasterIV"
#define IV_VALUE_TYPE_DEVICE_PZE_NAME                      "PZE-MasterIV"
#define IV_VALUE_TYPE_DEVICE_ZK_NAME                       "ZK-MasterIV"
#define IV_VALUE_TYPE_DEVICE_BDE_NAME                      "BDE-MasterIV"
#define IV_VALUE_TYPE_DEVICE_MOBIL_NAME                    "Mobil-MasterIV"
#define IV_VALUE_TYPE_DEVICE_MDE_NAME                      "MDE-BoxIV"
#define IV_VALUE_TYPE_DEVICE_AE_NAME                       "AE-MasterIV"
#define IV_VALUE_TYPE_DEVICE_TIMEBOY_NAME                  "TimeboyIV"
#define IV_VALUE_TYPE_DEVICE_EXKLUSIV_NAME                 "Exklusiv-Line IV"
#define IV_VALUE_TYPE_DEVICE_FLEX_NAME                     "Flex-MasterIV"
#define IV_VALUE_TYPE_DEVICE_TIMEBOY_MOBIL_PZE_NAME        "TimeboyIV Mobil-PZE"
#define IV_VALUE_TYPE_DEVICE_EVO_2_8_PURE_NAME             "EVO 2.8 Pure"
#define IV_VALUE_TYPE_DEVICE_EVO_4_3_NAME                  "EVO 4.3"
#define IV_VALUE_TYPE_DEVICE_IPC_EXTENSION_NAME            "IPC Embedded-System"
#define IV_VALUE_TYPE_DEVICE_IPC_EXTENSION_VARIO_5_7_NAME  "IPC-Extension A"
#define IV_VALUE_TYPE_DEVICE_IPC_EXTENSION_VARIO_10_NAME   "IPC-Extension B"
#define IV_VALUE_TYPE_DEVICE_IPC_EXTENSION_Q7_NAME         "IPC-Extension C"
#define IV_VALUE_TYPE_DEVICE_IO_BOX_NAME                   "IO-Box V4"
#define IV_VALUE_TYPE_DEVICE_KYO_INLOC_NAME                "KYO Inloc"
#define IV_VALUE_TYPE_DEVICE_MOBIL_BOX_NAME                "Mobil-Box V4"
#define IV_VALUE_TYPE_DEVICE_TIMEBOY_TBDOCK_NAME           "Dockingstation V2"
#define IV_VALUE_TYPE_DEVICE_MOBIL_TBDOCK_NAME             "Fahrzeugdatenlogger V2"
#define IV_VALUE_TYPE_DEVICE_KYO_ONELOC_NAME               "KYO Oneloc"
#define IV_VALUE_TYPE_DEVICE_EVO_3_5_UNIVERSAL_NAME        "EVO 3.5 Universal"
#define IV_VALUE_TYPE_DEVICE_SAMS_OCL_READER_NAME          "SAMS-OCL-Reader"
#define IV_VALUE_TYPE_DEVICE_EVO_3_5_PURE_NAME             "EVO 3.5 Pure"
#define IV_VALUE_TYPE_DEVICE_EVO_AGERA_NAME                "EVO Agera"
#define IV_VALUE_TYPE_DEVICE_KYO_CENLOC_NAME               "KYO Cenloc"
#define IV_VALUE_TYPE_DEVICE_VARIO_10_NAME                 "Vario 10"
#define IV_VALUE_TYPE_DEVICE_VARIO_12_NAME                 "Vario 12"
#define IV_VALUE_TYPE_DEVICE_VARIO_15_NAME                 "Vario 15"
#define IV_VALUE_TYPE_DEVICE_VARIO_17_NAME                 "Vario 17"
#define IV_VALUE_TYPE_DEVICE_IPC_EVO_12_1_NAME             "IPC EVO 12.1"
#define IV_VALUE_TYPE_DEVICE_IPC_EVO_15_NAME               "IPC EVO 15"
#define IV_VALUE_TYPE_DEVICE_IPC_EVO_18_5_NAME             "IPC EVO 18.5"
#define IV_VALUE_TYPE_DEVICE_IPC_EVO_24_NAME               "IPC EVO 24"
#define IV_VALUE_TYPE_DEVICE_EVO_INTERA_2_NAME             "EVO Intera 2"
#define IV_VALUE_TYPE_DEVICE_EVO_4_6_FLEX_KEY_NAME         "EVO 4.6 FlexKey"
#define IV_VALUE_TYPE_DEVICE_EVO_5_0_PURE_NAME             "EVO 5.0 Pure"

// Setzen/Ermitteln des Boardtyp (Redesignversion z.B.: 0x13) (in Nibbles)
#define IV_FUNCTION_SET_TYPE_BOARD               0x0003
#define IV_FUNCTION_GET_TYPE_BOARD               0x0004

#define IV_VALUE_TYPE_BOARD_1_0                  0x10
#define IV_VALUE_TYPE_BOARD_1_1                  0x11
#define IV_VALUE_TYPE_BOARD_1_2                  0x12
#define IV_VALUE_TYPE_BOARD_1_3                  0x13
#define IV_VALUE_TYPE_BOARD_1_4                  0x14
#define IV_VALUE_TYPE_BOARD_1_5                  0x15
#define IV_VALUE_TYPE_BOARD_1_6                  0x16
#define IV_VALUE_TYPE_BOARD_1_7                  0x17
#define IV_VALUE_TYPE_BOARD_1_8                  0x18
#define IV_VALUE_TYPE_BOARD_1_9                  0x19
#define IV_VALUE_TYPE_BOARD_2_0                  0x20
#define IV_VALUE_TYPE_BOARD_2_1                  0x21
#define IV_VALUE_TYPE_BOARD_2_2                  0x22
#define IV_VALUE_TYPE_BOARD_2_3                  0x23
#define IV_VALUE_TYPE_BOARD_2_4                  0x24
#define IV_VALUE_TYPE_BOARD_2_5                  0x25
#define IV_VALUE_TYPE_BOARD_2_6                  0x26
#define IV_VALUE_TYPE_BOARD_2_7                  0x27
#define IV_VALUE_TYPE_BOARD_3_0                  0x30
#define IV_VALUE_TYPE_BOARD_3_1                  0x31
#define IV_VALUE_TYPE_BOARD_3_4                  0x34
#define IV_VALUE_TYPE_BOARD_4_0                  0x40
#define IV_VALUE_TYPE_BOARD_4_1                  0x41
#define IV_VALUE_TYPE_BOARD_4_2                  0x42
#define IV_VALUE_TYPE_BOARD_4_3                  0x43
#define IV_VALUE_TYPE_BOARD_4_4                  0x44
#define IV_VALUE_TYPE_BOARD_4_5                  0x45
#define IV_VALUE_TYPE_BOARD_4_6                  0x46
#define IV_VALUE_TYPE_BOARD_4_7                  0x47
#define IV_VALUE_TYPE_BOARD_4_8                  0x48
#define IV_VALUE_TYPE_BOARD_4_9                  0x49


// Setzen/Ermitteln der Boardkomponente: RS485 fuer Kommunikation mit PC
#define IV_FUNCTION_SET_COMPONENT_RS485_0        0x0005
#define IV_FUNCTION_GET_COMPONENT_RS485_0        0x0006

// Setzen/Ermitteln der Boardkomponente: TCP/IP
#define IV_FUNCTION_SET_TYPE_TCPIP               0x0007
#define IV_FUNCTION_GET_TYPE_TCPIP               0x0008

#define IV_VALUE_TYPE_TCPIP_NONE                 0 // Kein TCP/IP
#define IV_VALUE_TYPE_TCPIP_XPORT                1 // Lantronix XPort
#define IV_VALUE_TYPE_TCPIP_MATCHPORT            2 // Lantronix MatchPort
#define IV_VALUE_TYPE_TCPIP_RS9110_N_11_24       3 // WLAN Redpine RS9110-N-11-24 Wird über typeModemMC gehandhabt.

// Setzen/Ermitteln der Boardkomponente: Modem MC35i/MC55
#define IV_FUNCTION_SET_TYPE_MODEM_MC            0x0009
#define IV_FUNCTION_GET_TYPE_MODEM_MC            0x000A

#define IV_VALUE_TYPE_MODEM_MC_NONE              0 // Kein GSM/GPRS
#define IV_VALUE_TYPE_MODEM_MC_39                1 // Siemens MC35i/MC39
#define IV_VALUE_TYPE_MODEM_MC_55                2 // Siemens MC55
#define IV_VALUE_TYPE_MODEM_RS9110_N_11_24       3 // WLAN Redpine RS9110-N-11-24
#define IV_VALUE_TYPE_MODEM_BG96                 4 // BG96 Modem

// Setzen/Ermitteln der Boardkomponente: RS485 fuer Zutrittskontrolle
#define IV_FUNCTION_SET_COMPONENT_RS485_1        0x000B
#define IV_FUNCTION_GET_COMPONENT_RS485_1        0x000C

// Setzen/Ermitteln des Transpondertyps
#define IV_FUNCTION_SET_TYPE_TRANSPONDER         0x000D
#define IV_FUNCTION_GET_TYPE_TRANSPONDER         0x000E

#define IV_VALUE_TYPE_TRANSPONDER_NONE                 0 // Kein Transponderleser
#define IV_VALUE_TYPE_TRANSPONDER_TSR30                1 // GIS TSR30
#define IV_VALUE_TYPE_TRANSPONDER_TSR32                2 // GIS TSR32
#define IV_VALUE_TYPE_TRANSPONDER_PRIMO100             3 // PHG Primo 100 (Legic Prime)
#define IV_VALUE_TYPE_TRANSPONDER_MIFARE               4 // Cavitec Mifare
#define IV_VALUE_TYPE_TRANSPONDER_IBUTTON_EXTERN       5 // Dallas iButton (Als separates Modul angebunden)
#define IV_VALUE_TYPE_TRANSPONDER_SMARTRELAIS          6 // SimonsVoss SmartRelais
#define IV_VALUE_TYPE_TRANSPONDER_NEDAP                7 // Nedap
#define IV_VALUE_TYPE_TRANSPONDER_IBUTTON_INTERN       8 // Dallas iButton (Fest auf Platine bestueckt)
#define IV_VALUE_TYPE_TRANSPONDER_HID_PROXKEY          9 // HID Proxy 4065
#define IV_VALUE_TYPE_TRANSPONDER_TWN3_MULTI_ISO      10 // Mifare Desfire (ISO14443 / ISO15693)
#define IV_VALUE_TYPE_TRANSPONDER_PRIMO130            11 // PHG Primo 130 (Legic Prime/Advant)
#define IV_VALUE_TYPE_TRANSPONDER_TSR21               12 // GIS TSR21
#define IV_VALUE_TYPE_TRANSPONDER_HID_ICLASS          13 // HID iClass OEM50
#define IV_VALUE_TYPE_TRANSPONDER_TB_MIFARE           14 // Mifare Lesemodul fuer TimeboyIV
#define IV_VALUE_TYPE_TRANSPONDER_TB_LEGIC            15 // Legic Lesemodul fuer TimeboyIV
#define IV_VALUE_TYPE_TRANSPONDER_SMARTRELAIS_G2      16 // SimonsVoss SmartRelais_G2
#define IV_VALUE_TYPE_TRANSPONDER_TWN3_MIFARE_NFC     17 // TWN3 Mifare NFC (ISO14443)
#define IV_VALUE_TYPE_TRANSPONDER_IDTRONIC_UHF        18 // ID Tronic UHF (long range)

// Setzen/Ermitteln des Displaytyp
#define IV_FUNCTION_SET_TYPE_DISPLAY             0x000F
#define IV_FUNCTION_GET_TYPE_DISPLAY             0x0010

#define IV_VALUE_TYPE_DISPLAY_NONE               0 // Kein Display
#define IV_VALUE_TYPE_DISPLAY_SMALL              1 // Lehner Dabitros (320 x 240) 4 Zoll Monochrom
#define IV_VALUE_TYPE_DISPLAY_BIG                2 // Lehner Dabitros (320 x 240) 6 Zoll Monochrom
#define IV_VALUE_TYPE_DISPLAY_SMALLER            3 // 1,9 Zoll Color
#define IV_VALUE_TYPE_DISPLAY_TIMEBOY_2ZOLL      4 // 2 Zoll Color Ilitek für TimeboyIV

// Setzen/Ermitteln des Tastaturtyps
#define IV_FUNCTION_SET_TYPE_KEYPAD              0x0011
#define IV_FUNCTION_GET_TYPE_KEYPAD              0x0012

#define IV_VALUE_TYPE_KEYPAD_NONE                0 // Keine Tastatur
#define IV_VALUE_TYPE_KEYPAD_9                   1 // 9 Tasten Tastatur (PZE-MasterIV usw.)
#define IV_VALUE_TYPE_KEYPAD_58                  2 // 56+2 Tasten Tastatur (BDE-MasterIV usw.)
#define IV_VALUE_TYPE_KEYPAD_29                  3 // 28+1 Tasten Tastatur (AE-MasterIV usw.)
#define IV_VALUE_TYPE_KEYPAD_7                   4 // 7 Tasten Tastatur (PZE-MasterIV Basic)
#define IV_VALUE_TYPE_KEYPAD_25                  5 // 25 Tasten Tastatur (TimeboyIV usw.)
#define IV_VALUE_TYPE_KEYPAD_20                  6 // 20 Tasten Tastatur (Flex-MasterIV usw.)
#define IV_VALUE_TYPE_KEYPAD_3                   7 // 3 Tasten (5 LED) Tastatur (Timeboy-Mobil-PZE usw.)

// Setzen/Ermitteln der Multimediakarte
#define IV_FUNCTION_SET_TYPE_MMC                 0x0013
#define IV_FUNCTION_GET_TYPE_MMC                 0x0014

#define IV_VALUE_TYPE_MMC_NONE                   0 // Keine MMC eingebaut
#define IV_VALUE_TYPE_MMC_EXTENSION              1 // Als Speichererweiterung
#define IV_VALUE_TYPE_MMC_DISK                   2 // Als Wechseldatentraeger
#define IV_VALUE_TYPE_MMC_USB                    3 // USB-Host als Speichererweiterung


// Setzen/Ermitteln des GPS - Typs
#define IV_FUNCTION_SET_TYPE_GPS                 0x0015
#define IV_FUNCTION_GET_TYPE_GPS                 0x0016

#define IV_VALUE_TYPE_GPS_NONE                   0 // Kein GPS
#define IV_VALUE_TYPE_GPS_GN80                   1 // Furuno GN-80
#define IV_VALUE_TYPE_GPS_5H                     2 // Lea 5H
#define IV_VALUE_TYPE_GPS_MOUSE_5H               3 // Navimouse mit Lea 5H


// Setzen/Ermitteln der USB - Komponente
#define IV_FUNCTION_SET_COMPONENT_USB            0x0017
#define IV_FUNCTION_GET_COMPONENT_USB            0x0018

// Setzen/Ermitteln der Hardwarekonfigurationsnummer
#define IV_FUNCTION_SET_HW_CONFIG                0x0019
#define IV_FUNCTION_GET_HW_CONFIG                0x001A

// Setzen/Ermitteln des Funkmodultyps
#define IV_FUNCTION_SET_TYPE_RADIO               0x001B
#define IV_FUNCTION_GET_TYPE_RADIO               0x001C

#define IV_VALUE_TYPE_RADIO_NONE                 0 // Kein Funk
#define IV_VALUE_TYPE_RADIO_WRAPTHOR             1 // Bluegiga Wrap Thor
#define IV_VALUE_TYPE_RADIO_ZIGBEE               2 // Zigbee


// Setzen/Ermitteln des Funkuhrmodultyps
#define IV_FUNCTION_SET_TYPE_DCF77               0x001D
#define IV_FUNCTION_GET_TYPE_DCF77               0x001E

#define IV_VALUE_TYPE_DCF77_NONE                 0 // Keine Funkuhr
#define IV_VALUE_TYPE_DCF77_HKW                  1 // HKW
#define IV_VALUE_TYPE_DCF77_BARCODE              2 // Barcodeleser von Timeboy intern
#define IV_VALUE_TYPE_DCF77_BARCODE_ZEBEX        3 // Barcodeleser Zebex CT-130 von Timeboy


// Setzen/Ermitteln der Option fuer analoge Eingaenge (32bit)
#define IV_FUNCTION_SET_COMPONENT_IN_ANALOG      0x001F
#define IV_FUNCTION_GET_COMPONENT_IN_ANALOG      0x0020

#define IV_VALUE_COMPONENT_IN_ANALOG_NONE        0x00000000
#define IV_VALUE_COMPONENT_IN_ANALOG_1           0x00000001
#define IV_VALUE_COMPONENT_IN_ANALOG_2           0x00000003


// Setzen/Ermitteln der Option fuer digitale Eingaenge (32bit)
#define IV_FUNCTION_SET_COMPONENT_IN_DIGITAL     0x0021
#define IV_FUNCTION_GET_COMPONENT_IN_DIGITAL     0x0022

#define IV_VALUE_COMPONENT_IN_DIGITAL_NONE       0x00000000
#define IV_VALUE_COMPONENT_IN_DIGITAL_1          0x00000001
#define IV_VALUE_COMPONENT_IN_DIGITAL_2          0x00000003
#define IV_VALUE_COMPONENT_IN_DIGITAL_3          0x00000007
#define IV_VALUE_COMPONENT_IN_DIGITAL_4          0x0000000F
#define IV_VALUE_COMPONENT_IN_DIGITAL_5          0x0000001F
#define IV_VALUE_COMPONENT_IN_DIGITAL_6          0x0000003F
#define IV_VALUE_COMPONENT_IN_DIGITAL_7          0x0000007F
#define IV_VALUE_COMPONENT_IN_DIGITAL_8          0x000000FF


// Setzen/Ermitteln der Option fuer digitale Ausgaenge (32bit)
#define IV_FUNCTION_SET_COMPONENT_OUT_DIGITAL    0x0023
#define IV_FUNCTION_GET_COMPONENT_OUT_DIGITAL    0x0024

#define IV_VALUE_COMPONENT_OUT_DIGITAL_NONE      0x00000000
#define IV_VALUE_COMPONENT_OUT_DIGITAL_1         0x00000001
#define IV_VALUE_COMPONENT_OUT_DIGITAL_2         0x00000003
#define IV_VALUE_COMPONENT_OUT_DIGITAL_3         0x00000007
#define IV_VALUE_COMPONENT_OUT_DIGITAL_4         0x0000000F


// Setzen/Ermitteln des Fingerprinttyps
#define IV_FUNCTION_SET_TYPE_FINGERPRINT         0x0025
#define IV_FUNCTION_GET_TYPE_FINGERPRINT         0x0026

#define IV_VALUE_TYPE_FINGERPRINT_NONE           0
#define IV_VALUE_TYPE_FINGERPRINT_BIOKEY_3000    1 // Idencom (BioKey 3000 - Modul)
#define IV_VALUE_TYPE_FINGERPRINT_BIOKEY_4000    2 // Idencom (BioKey 4000 - Modul)
#define IV_VALUE_TYPE_FINGERPRINT_BIOKEY_4020    3 // Idencom (BioKey 4020 - Modul)

// Setzen/Ermitteln der Typen der Externen Lesermodule
#define IV_FUNCTION_SET_TYPE_EXTERNAL_READER     0x0027
#define IV_FUNCTION_GET_TYPE_EXTERNAL_READER     0x0028

#define IV_VALUE_TYPE_EXTERNAL_READER_UNDEFINED  0x00 // Undefined
#define IV_VALUE_TYPE_EXTERNAL_READER_GIS        0x01 // GIS
#define IV_VALUE_TYPE_EXTERNAL_READER_PHG        0x02 // PHG
#define IV_VALUE_TYPE_EXTERNAL_READER_PROTECT    0x04 // Protect
#define IV_VALUE_TYPE_EXTERNAL_READER_IDTRONIC   0x08 // ID Tronic UHF

// Setzen/Ermitteln der Optionen fuer die Firmware
#define IV_FUNCTION_SET_OPTION_IOMODULE          0x0029
#define IV_FUNCTION_GET_OPTION_IOMODULE          0x002A

#define IV_VALUE_OPTION_IOMODULE_UNDEFINED       0xFF // Undefiniert
#define IV_VALUE_OPTION_IOMODULE_NONE            0x00 // Keines
#define IV_VALUE_OPTION_IOMODULE                 0x01 // IOModule Anbindung benoetigt

// Setzen/Ermitteln der Optionen fuer die Firmware
#define IV_FUNCTION_SET_OPTION_COMSERVER         0x002B
#define IV_FUNCTION_GET_OPTION_COMSERVER         0x002C

#define IV_VALUE_OPTION_COMSERVER_UNDEFINED      0xFF // Undefiniert
#define IV_VALUE_OPTION_COMSERVER_NONE           0x00 // Keines
#define IV_VALUE_OPTION_COMSERVER                0x01 // ComServerMode wird benoetigt

// Setzen/Ermitteln der Optionen fuer die Firmware
#define IV_FUNCTION_SET_OPTION_TBADAPTION        0x002D
#define IV_FUNCTION_GET_OPTION_TBADAPTION        0x002E

#define IV_VALUE_OPTION_TBADAPTION_UNDEFINED     0xFF // Undefiniert
#define IV_VALUE_OPTION_TBADAPTION_NONE          0x00 // Keines
#define IV_VALUE_OPTION_TBADAPTION               0x01 // ComServerMode wird benoetigt

// Ausfuehren von Befehlen fuer den Pruefplatz
#define IV_FUNCTION_INSPECTION_STATION           0x002F
#define IV_FUNCTION_INSPECTION_STATION_OK        0x0030

// Setzen/Ermitteln der Optionen fuer die Firmware
#define IV_FUNCTION_SET_OPTION_TPWRITE           0x0031
#define IV_FUNCTION_GET_OPTION_TPWRITE           0x0032

#define IV_VALUE_OPTION_TPWRITE_UNDEFINED        0xFF // Undefiniert
#define IV_VALUE_OPTION_TPWRITE_NONE             0x00 // Keines
#define IV_VALUE_OPTION_TPWRITE                  0x01 // wird benoetigt

// Setzen/Ermitteln der Optionen fuer die Firmware
#define IV_FUNCTION_SET_OPTION_HTTP              0x0033
#define IV_FUNCTION_GET_OPTION_HTTP              0x0034

#define IV_VALUE_OPTION_HTTP_UNDEFINED           0xFF // Undefiniert
#define IV_VALUE_OPTION_HTTP_NONE                0x00 // Keines
#define IV_VALUE_OPTION_HTTP                     0x01 // wird benoetigt

// Setzen/Ermitteln der Optionen fuer die Firmware
#define IV_FUNCTION_SET_OPTION_LANGUAGE          0x0035
#define IV_FUNCTION_GET_OPTION_LANGUAGE          0x0036

#define IV_VALUE_OPTION_LANGUAGE_UNDEFINED       0xFF // Undefiniert
#define IV_VALUE_OPTION_LANGUAGE_NONE            0x00 // Keines
#define IV_VALUE_OPTION_LANGUAGE                 0x01 // wird benoetigt

// Setzen/Ermitteln der Optionen fuer die Firmware
#define IV_FUNCTION_SET_OPTION_BIOKEY_UPDATE     0x0037
#define IV_FUNCTION_GET_OPTION_BIOKEY_UPDATE     0x0038

#define IV_VALUE_OPTION_BIOKEY_UPDATE_UNDEFINED  0xFF // Undefiniert
#define IV_VALUE_OPTION_BIOKEY_UPDATE_NONE       0x00 // Keines
#define IV_VALUE_OPTION_BIOKEY_UPDATE            0x01 // wird benoetigt

// Setzen/Ermitteln der Optionen fuer das Setup
#define IV_FUNCTION_SET_SETUP_OPTIONS            0x0039
#define IV_FUNCTION_GET_SETUP_OPTIONS            0x003A

#define IV_VALUE_SETUP_OPTION_MASK               0x0001 // Maske der unterstützten Setupoptionen
#define IV_VALUE_SETUP_OPTION_ALL                0xFFFF
#define IV_VALUE_SETUP_OPTION_IOCONTROL          0x0001 // Signalverarbeitung

// Setzen/Ermitteln des Touchtyp
#define IV_FUNCTION_SET_TYPE_TOUCH               0x003B
#define IV_FUNCTION_GET_TYPE_TOUCH               0x003C

#define IV_VALUE_TYPE_TOUCH_NONE                 0 // Kein Touch
#define IV_VALUE_TYPE_TOUCH_SMALL                1 // 4 Zoll Touchpanel
#define IV_VALUE_TYPE_TOUCH_BIG                  2 // 6 Zoll Touchpanel

// Setzen/Ermitteln des Sensortyp
#define IV_FUNCTION_SET_TYPE_SENSOR              0x003D
#define IV_FUNCTION_GET_TYPE_SENSOR              0x003E

#define IV_VALUE_TYPE_SENSOR_NONE                 0 // Kein Sensor
#define IV_VALUE_TYPE_SENSOR_VIBRATION            1 // Vibrationssensor bestückt
#define IV_VALUE_TYPE_SENSOR_SPEEDUP              2 // Beschleunigungssensor bestückt

// Setzen/Ermitteln der Komponente Lithium Batterie bestückt
#define IV_FUNCTION_SET_COMPONENT_BATTERY        0x003F
#define IV_FUNCTION_GET_COMPONENT_BATTERY        0x0040

// Setzen/Ermitteln der Komponente Ladeschaltung bestückt
#define IV_FUNCTION_SET_COMPONENT_CHARGE         0x0041
#define IV_FUNCTION_GET_COMPONENT_CHARGE         0x0042

// Setzen/Ermitteln eines Extension Bords
//#define IV_FUNCTION_SET_EXTENSION_BOARDS         0x0043
//#define IV_FUNCTION_GET_EXTENSION_BORADS         0x0044

// Ermitteln der gesamten Hardwareinfo in einer einzelnen Struktur
#define IV_FUNCTION_GET_HARDWARE_INFO            0x0045

//#############################################################################
// Da es kein Protokollmodul gibt werden die wenigen benoetigten Funktionen 
// aus dem Hardwaremodul heraus genommen. Das Define ist in dem Protocolmodul zu suchen.
//#############################################################################
//#define IV_FUNCTION_EXCHANGE_PUBLIC_KEY          0x0046

// Setzen/Ermitteln der Bezahlbaren Optionen
#define IV_FUNCTION_SET_PAYABLE_OPTIONS          0x0047
#define IV_FUNCTION_GET_PAYABLE_OPTIONS          0x0048

#define IV_VALUE_PAYABLE_OPTION_MASK             0x03 // Maske der unterstützten Optionen
#define IV_VALUE_PAYABLE_OPTION_ALL              0xFF
#define IV_VALUE_PAYABLE_OPTION_DATA_ON_CARD     0x01 // Data on Card
#define IV_VALUE_PAYABLE_OPTION_EXTENDED_FLASH   0x02 // Erweiterter Flashspeicher
//BO_15.04.2016: Der Wert darf nie 0xFF werden, da sonst eine Intialisierung auf 0 in der FW durchgefuehrt wird.

namespace Datafox
{
    namespace Communication
    {
        static const std::map<uint8_t, std::string> g_deviceNames{
            { IV_VALUE_TYPE_DEVICE_PZE,                         IV_VALUE_TYPE_DEVICE_PZE_NAME }, 
            { IV_VALUE_TYPE_DEVICE_ZK,                          IV_VALUE_TYPE_DEVICE_ZK_NAME }, 
            { IV_VALUE_TYPE_DEVICE_BDE,                         IV_VALUE_TYPE_DEVICE_BDE_NAME },
            { IV_VALUE_TYPE_DEVICE_MOBIL,                       IV_VALUE_TYPE_DEVICE_MOBIL_NAME },
            { IV_VALUE_TYPE_DEVICE_MDE,                         IV_VALUE_TYPE_DEVICE_MDE_NAME },
            { IV_VALUE_TYPE_DEVICE_AE,                          IV_VALUE_TYPE_DEVICE_AE_NAME },
            { IV_VALUE_TYPE_DEVICE_TIMEBOY,                     IV_VALUE_TYPE_DEVICE_TIMEBOY_NAME },
            { IV_VALUE_TYPE_DEVICE_EXKLUSIV,                    IV_VALUE_TYPE_DEVICE_EXKLUSIV_NAME },
            { IV_VALUE_TYPE_DEVICE_FLEX,                        IV_VALUE_TYPE_DEVICE_FLEX_NAME },
            { IV_VALUE_TYPE_DEVICE_TIMEBOY_MOBIL_PZE,           IV_VALUE_TYPE_DEVICE_TIMEBOY_MOBIL_PZE_NAME },
            { IV_VALUE_TYPE_DEVICE_EVO_2_8_PURE,                IV_VALUE_TYPE_DEVICE_EVO_2_8_PURE_NAME },
            { IV_VALUE_TYPE_DEVICE_EVO_4_3,                     IV_VALUE_TYPE_DEVICE_EVO_4_3_NAME },
            { IV_VALUE_TYPE_DEVICE_IPC_EXTENSION_VARIO_5_7,     IV_VALUE_TYPE_DEVICE_IPC_EXTENSION_VARIO_5_7_NAME },
            { IV_VALUE_TYPE_DEVICE_IPC_EXTENSION_VARIO_10,      IV_VALUE_TYPE_DEVICE_IPC_EXTENSION_VARIO_10_NAME },
            { IV_VALUE_TYPE_DEVICE_IPC_EXTENSION_Q7,            IV_VALUE_TYPE_DEVICE_IPC_EXTENSION_Q7_NAME },
            { IV_VALUE_TYPE_DEVICE_IO_BOX,                      IV_VALUE_TYPE_DEVICE_IO_BOX_NAME },
            { IV_VALUE_TYPE_DEVICE_KYO_INLOC,                   IV_VALUE_TYPE_DEVICE_KYO_INLOC_NAME },
            { IV_VALUE_TYPE_DEVICE_MOBIL_BOX,                   IV_VALUE_TYPE_DEVICE_MOBIL_BOX_NAME },
            { IV_VALUE_TYPE_DEVICE_TIMEBOY_TBDOCK,              IV_VALUE_TYPE_DEVICE_TIMEBOY_TBDOCK_NAME },
            { IV_VALUE_TYPE_DEVICE_MOBIL_TBDOCK,                IV_VALUE_TYPE_DEVICE_MOBIL_TBDOCK_NAME },
            { IV_VALUE_TYPE_DEVICE_KYO_ONELOC,                  IV_VALUE_TYPE_DEVICE_KYO_ONELOC_NAME },
            { IV_VALUE_TYPE_DEVICE_EVO_3_5_UNIVERSAL,           IV_VALUE_TYPE_DEVICE_EVO_3_5_UNIVERSAL_NAME },
            { IV_VALUE_TYPE_DEVICE_SAMS_OCL_READER,             IV_VALUE_TYPE_DEVICE_SAMS_OCL_READER_NAME },
            { IV_VALUE_TYPE_DEVICE_EVO_3_5_PURE,                IV_VALUE_TYPE_DEVICE_EVO_3_5_PURE_NAME },
            { IV_VALUE_TYPE_DEVICE_EVO_AGERA,                   IV_VALUE_TYPE_DEVICE_EVO_AGERA_NAME },
            { IV_VALUE_TYPE_DEVICE_KYO_CENLOC,                  IV_VALUE_TYPE_DEVICE_KYO_CENLOC_NAME },
            { IV_VALUE_TYPE_DEVICE_VARIO_10,                    IV_VALUE_TYPE_DEVICE_VARIO_10_NAME },
            { IV_VALUE_TYPE_DEVICE_VARIO_12,                    IV_VALUE_TYPE_DEVICE_VARIO_12_NAME },
            { IV_VALUE_TYPE_DEVICE_VARIO_15,                    IV_VALUE_TYPE_DEVICE_VARIO_15_NAME },
            { IV_VALUE_TYPE_DEVICE_VARIO_17,                    IV_VALUE_TYPE_DEVICE_VARIO_17_NAME },
            { IV_VALUE_TYPE_DEVICE_IPC_EVO_12_1,                IV_VALUE_TYPE_DEVICE_IPC_EVO_12_1_NAME },
            { IV_VALUE_TYPE_DEVICE_IPC_EVO_15,                  IV_VALUE_TYPE_DEVICE_IPC_EVO_15_NAME },
            { IV_VALUE_TYPE_DEVICE_IPC_EVO_18_5,                IV_VALUE_TYPE_DEVICE_IPC_EVO_18_5_NAME },
            { IV_VALUE_TYPE_DEVICE_IPC_EVO_24,                  IV_VALUE_TYPE_DEVICE_IPC_EVO_24_NAME },
            { IV_VALUE_TYPE_DEVICE_EVO_INTERA_2,                IV_VALUE_TYPE_DEVICE_EVO_INTERA_2_NAME },
            { IV_VALUE_TYPE_DEVICE_EVO_4_6_FLEX_KEY,            IV_VALUE_TYPE_DEVICE_EVO_4_6_FLEX_KEY_NAME },
            { IV_VALUE_TYPE_DEVICE_EVO_5_0_PURE,                IV_VALUE_TYPE_DEVICE_EVO_5_0_PURE_NAME }
        };

        class ModuleHardware : public ModuleBase
        {
        public:
            ModuleHardware( int32_t connectionNumber = ConnectionBase::defaultConnectionNumber(), uint8_t deviceAddress = ProtocolBase::defaultDeviceAddress() );
            virtual ~ModuleHardware();

        public:
            // Gibt Modulfehler zurueck.
            virtual void errorModule( uint16_t module, uint16_t error, std::string &text );
            // Gibt Funktionsfehlertext zurueck.
            virtual void errorFunction( uint16_t module, uint16_t func, uint16_t error, std::string &text );

            // Einfache Anfrage ob Geraet mit Protokollversion 4 erreichbar.
            //int32_t request( uint8_t deviceAddress );

            // MODULE_HARDWARE

            // Einfache Anfrage ob das Geraet antwortet.
            int32_t randomPing( uint8_t deviceAddress );

            // BUG:
            // Im Bootloader 00.04.00.00 && 00.04.00.01 liefert die Funktion
            // - TypeTransponder den Wert fuer das Display
            // - TypeDisplay den Wert fuer das Keypad
            // - TypeKeypad den Wert fuer den Transponder

            // Typ schreiben/lesen
            int32_t writeDeviceType( uint8_t deviceAddress, uint8_t val, uint16_t val2 = 0xFFFF );
            int32_t readDeviceType( uint8_t deviceAddress, uint8_t &val );
            int32_t writeBoardType( uint8_t deviceAddress, uint8_t val, uint16_t val2 = 0xFFFF );
            int32_t readBoardType( uint8_t deviceAddress, uint8_t &val );
            int32_t writeNetworkAdapterType( uint8_t deviceAddress, uint8_t val, uint16_t val2 = 0xFFFF );
            int32_t readNetworkAdapterType( uint8_t deviceAddress, uint8_t &val );
            int32_t writeModemType( uint8_t deviceAddress, uint8_t val, uint16_t val2 = 0xFFFF );
            int32_t readModemType( uint8_t deviceAddress, uint8_t &val );
            int32_t writeDisplayType( uint8_t deviceAddress, uint8_t val, uint16_t val2 = 0xFFFF, bool checked = false );
            int32_t readDisplayType( uint8_t deviceAddress, uint8_t &val, bool checked = false );
            int32_t writeKeypadType( uint8_t deviceAddress, uint8_t val, uint16_t val2 = 0xFFFF, bool checked = false );
            int32_t readKeypadType( uint8_t deviceAddress, uint8_t &val, bool checked = false );
            int32_t writeTransponderReaderType( uint8_t deviceAddress, uint8_t val, uint16_t val2 = 0xFFFF, bool checked = false );
            int32_t readTransponderReaderType( uint8_t deviceAddress, uint8_t &val, bool checked = false );
            int32_t writeMmcType( uint8_t deviceAddress, uint8_t val, uint16_t val2 = 0xFFFF );
            int32_t readMmcType( uint8_t deviceAddress, uint8_t &val );
            int32_t writeRadioType( uint8_t deviceAddress, uint8_t val, uint16_t val2 = 0xFFFF );
            int32_t readRadioType( uint8_t deviceAddress, uint8_t &val );
            int32_t writeGpsType( uint8_t deviceAddress, uint8_t val, uint16_t val2 = 0xFFFF );
            int32_t readGpsType( uint8_t deviceAddress, uint8_t &val );
            int32_t writeDCF77Type( uint8_t deviceAddress, uint8_t val, uint16_t val2 = 0xFFFF );
            int32_t readDCF77Type( uint8_t deviceAddress, uint8_t &val );
            int32_t writeFingerprintScannerType( uint8_t deviceAddress, uint8_t val, uint16_t val2 = 0xFFFF );
            int32_t readFingerprintScannerType( uint8_t deviceAddress, uint8_t &val );
            int32_t writeExternalTransponderReaderType( uint8_t deviceAddress, uint8_t val, uint16_t val2 = 0xFFFF );
            int32_t readExternalTransponderReaderType( uint8_t deviceAddress, uint8_t &val );
            int32_t writeTouchscreenType( uint8_t deviceAddress, uint8_t val, uint16_t val2 = 0xFFFF );
            int32_t readTouchscreenType( uint8_t deviceAddress, uint8_t &val );
            int32_t writeSensorType( uint8_t deviceAddress, uint8_t val, uint16_t val2 = 0xFFFF );
            int32_t readSensorType( uint8_t deviceAddress, uint8_t &val );

            // Komponente schreiben/lesen
            int32_t writeRs485CommunicationComponent( uint8_t deviceAddress, uint8_t val, uint16_t val2 = 0xFFFF );
            int32_t readRs485CommunicationComponent( uint8_t deviceAddress, uint8_t &val );
            int32_t writeRs485AccessControlComponent( uint8_t deviceAddress, uint8_t val, uint16_t val2 = 0xFFFF );
            int32_t readRs485AccessControlComponent( uint8_t deviceAddress, uint8_t &val );
            int32_t writeUsbComponent( uint8_t deviceAddress, uint8_t val, uint16_t val2 = 0xFFFF );
            int32_t readUsbComponent( uint8_t deviceAddress, uint8_t &val );
            int32_t writeAnalogInputComponent( uint8_t deviceAddress, uint32_t val, uint16_t val2 = 0xFFFF );
            int32_t readAnalogInputComponent( uint8_t deviceAddress, uint32_t& val );
            int32_t writeDigitalInputComponent( uint8_t deviceAddress, uint32_t val, uint16_t val2 = 0xFFFF );
            int32_t readDigitalInputComponent( uint8_t deviceAddress, uint32_t& val );
            int32_t writeDigitalOutputComponent( uint8_t deviceAddress, uint32_t val, uint16_t val2 = 0xFFFF );
            int32_t readDigitalOutputComponent( uint8_t deviceAddress, uint32_t& val );
            int32_t writeBatteryComponent( uint8_t deviceAddress, uint8_t val, uint16_t val2 = 0xFFFF );
            int32_t readBatteryComponent( uint8_t deviceAddress, uint8_t &val );
            int32_t writeBatteryChargerComponent( uint8_t deviceAddress, uint8_t val, uint16_t val2 = 0xFFFF );
            int32_t readBatteryChargerComponent( uint8_t deviceAddress, uint8_t &val );

            // Optionen schreiben/lesen
            int32_t writeIoModuleOption( uint8_t deviceAddress, uint8_t val, uint16_t val2 = 0xFFFF );
            int32_t readIoModuleOption( uint8_t deviceAddress, uint8_t &val );
            int32_t writeComServerOption( uint8_t deviceAddress, uint8_t val, uint16_t val2 = 0xFFFF );
            int32_t readComServerOption( uint8_t deviceAddress, uint8_t &val );
            int32_t writeTimeboyAdaptionOption( uint8_t deviceAddress, uint8_t val, uint16_t val2 = 0xFFFF );
            int32_t readTimeboyAdaptionOption( uint8_t deviceAddress, uint8_t &val );
            int32_t writeTransponderWriteOption( uint8_t deviceAddress, uint8_t val, uint16_t val2 = 0xFFFF );
            int32_t readTransponderWriteOption( uint8_t deviceAddress, uint8_t &val );
            int32_t writeHttpOption( uint8_t deviceAddress, uint8_t val, uint16_t val2 = 0xFFFF );
            int32_t readHttpOption( uint8_t deviceAddress, uint8_t &val );
            int32_t writeLocalizationOption( uint8_t deviceAddress, uint8_t val, uint16_t val2 = 0xFFFF );
            int32_t readLocalizationOption( uint8_t deviceAddress, uint8_t &val );
            int32_t writeFingerprintScannerUpdateOption( uint8_t deviceAddress, uint8_t val, uint16_t val2 = 0xFFFF );
            int32_t readFingerprintScannerUpdateOption( uint8_t deviceAddress, uint8_t &val );
            int32_t writeSetupOptions( uint8_t deviceAddress, uint16_t mask, uint16_t options );
            int32_t readSetupOptions( uint8_t deviceAddress, uint16_t &mask, uint16_t &options );
            int32_t writeBuyOptions( uint8_t deviceAddress, uint8_t mask, uint8_t options, uint16_t val2 = 0xFFFF );
            int32_t readBuyOptions( uint8_t deviceAddress, uint8_t &mask, uint8_t &options );

            bool isMasterIV( const StructHardwareInfo &info )
            {
                bool res = false;

                // Gerätetyp muss einer dieser sein.
                switch ( info.typeDevice )
                {
                    case IV_VALUE_TYPE_DEVICE_PZE:
                    case IV_VALUE_TYPE_DEVICE_ZK:
                    case IV_VALUE_TYPE_DEVICE_BDE:
                    case IV_VALUE_TYPE_DEVICE_MOBIL:
                    case IV_VALUE_TYPE_DEVICE_MDE:
                    case IV_VALUE_TYPE_DEVICE_AE:
                    case IV_VALUE_TYPE_DEVICE_TIMEBOY:
                    case IV_VALUE_TYPE_DEVICE_EXKLUSIV:
                    case IV_VALUE_TYPE_DEVICE_FLEX:
                    case IV_VALUE_TYPE_DEVICE_TIMEBOY_MOBIL_PZE:
                        // Zusätzlich muss die Boardversion kleiner 4.0 sein.
                        if ( info.typeBoard < 0x40 )
                        {
                            res = true;
                        }
                }

                return res;
            }

            // Liest die gesamte Hardwareinfo struktur aus.
            int32_t readHardwareInfo( uint8_t deviceAddress, StructHardwareInfo& hardware_info );

            // HWConfigversion schreiben/lesen
            int32_t writeHardwareInfoVersion( uint8_t deviceAddress, uint8_t val, uint16_t val2 = 0xFFFF );
            int32_t readHardwareInfoVersion( uint8_t deviceAddress, uint8_t &val );

            // Befehl an Pruefplatz senden
            int32_t inspectionStationCommand( uint8_t deviceAddress, uint8_t address, char *data, int32_t sizeData, uint8_t& addressAnswer, char *dataAnswer, int32_t &sizeAnswer, int32_t timeoutPending );
            int32_t replyInspectionStationCommand( uint8_t deviceAddress );

            //#################################
            // CallRoutinen

            // Ausfuehren von Befehlen
            virtual int32_t remoteCall( uint16_t module, uint16_t func, uint8_t *data, uint16_t &count );

        public:
            StructHardwareInfo m_hardwareInfo;
        };
    } // namespace Communication
} // namespace Datafox

#endif
