#include "file/dfcom_ini_helper.h"
#if defined(_MSC_VER) || defined(__MINGW32__)
#ifndef WIN32_LEAN_AND_MEAN
#define WIN32_LEAN_AND_MEAN
#endif
#include <windows.h>
#include <direct.h>
#include <io.h>
#endif

#if !defined(_MSC_VER)
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <dirent.h>
#endif

#include <fstream>
#include <boost/algorithm/string.hpp>

bool DFComIniHelper::createIniFile( const std::string &file_path, const std::string &log_base_path )
{
    std::ofstream out( file_path );
    // Wurde die Datei gültig erstellt.
    if ( out.is_open() )
    {
        // Sektion "Log" anlegen
        out << "[Log]" << std::endl;
#if !defined(_MSC_VER) && !defined(__MINGW32__)
        out << "Enable=0" << std::endl; // Unter Linux ist das Loggen standardmaessig aus.
        out << "; ACHTUNG: Das Loggen ist deaktiviert, da der Key -Erase- aktuell unter GNU/Linux nicht unterstuetzt wird." << std::endl;
        out << ";          Bei Aktivierung liegt das Loeschen der Logdateien in Ihren Haenden." << std::endl;
#endif
#if defined(_MSC_VER) || defined(__MINGW32__)
        out << "Enable=1" << std::endl;
#endif
        out << "Path=" << log_base_path << std::endl;
        out << "Folder=" << SUB_FOLDER << std::endl;
        out << "Trace=0" << std::endl;
        out << "Erase=1" << std::endl;
        out << "Keep=10" << std::endl;
        out << "Elapse=1" << std::endl;
        out << "List=1" << std::endl;

        out << std::endl << std::endl;
        out << "; Kurzbeschreibung der enthaltenen Parameter:" << std::endl;
        out << "; Enable: Aktivieren des Hintergrundloggens. Wertebereich: 0 = Nicht loggen, 1 = Loggen aktiviert." << std::endl;
        out << "; Path: Gibt das Basisverzeichnis an. Bei Verwendung von Backslash-Zeichen \\ muessen diese doppelt eingegeben werden." << std::endl;
        out << "; Folder: Das zu verwendende Unterverzeichnis, dieses wird wenn nicht bereits vorhanden angelegt." << std::endl;
        out << "; Trace: Gibt an ob das DLL eigene Detaillog aktiviert sein soll. Wertebereich: 0 = Deaktiviert, 1 = Aktiviert, 2 = Aktiviert mit Packetdaten.\n; \tBitte beachten Sie, dass hierbei groessere Datenmengen anfallen koennen." << std::endl;
        out << "; Erase: Gibt an ob die Logdateien automatisch geloescht werden, unter Linux werden die Logdateien nicht automatisch geloescht. Wertebreich: 1 = Automatisch loeschen, 0 = Manuell loeschen.\n; \tBitte beachten Sie dass die Dateiverwaltung nach Abschalten uebernommen werden muss." << std::endl;
        out << "; Keep: Gibt bei Erase=1 die Anzahl der zu haltenden Logtage an (nicht unter Linux gueltig). Wertebereich: 1 - 100 Logtage. Logtage bedeutet hier, Tage an denen geloggt wurde." << std::endl;
        out << "; Elapse: Gibt an ob die Ausfuehrungsdauer der jeweiligen Funktion in eckigen Klammern hinter jedem Logeintrag ausgegeben wird, dabei hat dieser folgende Aufteilung [Sekunden.Hundertstel]." << std::endl;
        out << "; List: Gibt an ob die uebertragenen Listendaten vor jedem uebertragen, in das Logverzeichnis exportiert werden.\n; \tWertebereich: 0 = Nein, 1 = In Binaerformat mitprotokollieren, 2 = In Binaerform und Textformat (tab-Getrennt)." << std::endl;

        // Sektion "Docking" anlegen
        out << std::endl << std::endl;
        out << "[Docking]" << std::endl;
        out << "Prefer=2" << std::endl;
        //out << "Autodetect=1" << std::endl;

        out << std::endl << std::endl;
        out << "; Kurzbeschreibung der enthaltenen Parameter:" << std::endl;
        out << "; Sektion [Docking] ist global gültig. Für eine Verbindungsspezifische Einstellung verwenden Sie bitte" << std::endl;
        out << "; einen Sektionsnamen [Docking_connectionId] z. B. [Docking_5] für die Verbindungs-Id 5." << std::endl;
        out << "; Prefer: Bevorzugte Version der Dockingstation. Wertebereich: 1 = Version 1, 2 = Version 2." << std::endl;
        //out << "; Autodetect: Automatisch die Version der Dockingstation ermitteln, angefangen wird die Ermittlung mit dem Wert von Prefer. Wertebereich: 0 = Aus, 1 = An." << std::endl;

        // Sektion "Docking" anlegen
        out << std::endl << std::endl;
        out << "[Channels]" << std::endl;
        out << "CheckOpenCloseBySameThread=1" << std::endl;
        //out << "Autodetect=1" << std::endl;

        out << std::endl << std::endl;
        out << "; Kurzbeschreibung der enthaltenen Parameter:" << std::endl;
        out << "; Sektion [Channels] ist global gueltig für das Verhalten der Verbindungsklassen." << std::endl;
        out << "; CheckOpenCloseBySameThread: Das Oeffnen und Schliessen der Verbindung, muss durch den selben Thread erfolgen. Wertebereich: 0 = Deaktiviert, 1 = Aktiviert (Default)." << std::endl;

        // Sektion "XML" anlegen
        out << std::endl << std::endl;
        out << "[XML]" << std::endl;
        out << "createHelpFile=0" << std::endl;
        out << "docPath=" << std::endl;

        out << std::endl << std::endl;
        out << "; Kurzbeschreibung der enthaltenen Parameter:" << std::endl;
        out << "; Die Sektion [XML] beinhaltet Einstellung im Bezug auf die XML-Konfiguration." << std::endl;
        out << "; createHelpFile: Aktiviert die Erzeugung der Hilfedatei, wenn die Konfiguration als XML gespeichert oder zu XML konvertiert wird. Wertebereich: 0 = Deaktiviert (Default), 1 = Aktiviert" << std::endl;
        out << "; docPath: Die erzeugte Hilfedatei (html-Datei) arbeitet mit Verweisen auf die DFCom-Dokumentation. Damit diese funktionieren muss der Pfad zur Dokumentation bekannt gemacht werden." << std::endl;

        // Sektion "DeviceUpdate" anlegen
        out << std::endl << std::endl;
        out << "[DeviceUpdate]" << std::endl;
        out << "dbPath=" << std::endl;
        out << "dbPathType=0" << std::endl;

        out << std::endl << std::endl;
        out << "; Kurzbeschreibung der enthaltenen Parameter:" << std::endl;
        out << "; Die Sektion [DeviceUpdate] beinhaltet Einstellung im Bezug auf die Geräteaktualisierung." << std::endl;
        out << "; dbPath: Die Prüfung der Abhängigkeit zwischen Firmware und Gerät erfolgt auf Grundlage der verbauten Hardwaremodule. Die Information dazu wird in einer Datenbank abgelegt. Hier wird der Pfad zur Datenbankdatei angegeben." << std::endl;
        out << "; dbPathType: Definiert den Type der Pfadangabe. Wertebereich: 0 = undefiniert (Default), 1 = Lokal, 2 = URI" << std::endl;
    }

    return out.is_open();
}

std::string DFComIniHelper::findIniFile( bool create_if_not_exist )
{
    std::string ini_file_path;
    // Suchen der Initialisierungsdatei.
    // 1. Bei dem Executable.
    // 2. Umgebungsvariable DFCOM_INI_PATH
    // 3. Umgebungsvariable LOCALAPPDATA unter Windows oder HOME unter GNU/Linux.

    // 1. Versuchen die DFCom.ini in dem Verzeichnis der Anwendung zu finden.
    std::string executable_file_path( 260, '\0' );

#if defined(_MSC_VER) || defined(__MINGW32__)
    // Datei der Anwendung
    if ( GetModuleFileNameA( nullptr, &executable_file_path[0], (DWORD)executable_file_path.size() ) == 0 )
    {
        // Fehler bei Ermittlung
        executable_file_path.clear();
    }
    else
    {
        // Windows akzeptiert auch '/' als Pfadtrenner, somit Gemeinsamkeit mit GNU/Linux herstellen.
        boost::replace_all( executable_file_path, "\\", "/" );
    }
#else
    // Datei der Anwendung
    std::string tmp( 260, '\0' );
    std::sprintf( &tmp[0], "/proc/%d/exe", getpid() );
    if ( readlink( tmp.c_str(), &executable_file_path[0], executable_file_path.size() ) == -1 )
    {
        // Fehler bei Ermittlung
        executable_file_path.clear();
    }
#endif
    // Prüfen ob Inidatei bei Anwendung existiert.
    if ( executable_file_path.size() )
    {
        auto pos = executable_file_path.find_first_of( '\0' );
        if ( pos != std::string::npos )
        {
            // Stringgröße an Inhalt anpassen.
            executable_file_path.resize( pos );
            pos = executable_file_path.find_last_of( '/' );
            if ( pos != std::string::npos )
            {
                ini_file_path = executable_file_path;
                ini_file_path.resize( pos + 1 );
                ini_file_path += "DFCom.ini";

                // Prüfen ob die Datei dort existiert.
                struct stat lstat;
                if ( stat( ini_file_path.c_str(), &lstat ) == -1 || (lstat.st_mode & S_IFMT) != S_IFREG )
                {
                    ini_file_path.clear();
                }
            }
        }
    }

    if ( !ini_file_path.size() )
    {
        // 2. Versuchen die DFCom.ini über die Umgebungsvariable DFCOM_INI_PATH zu finden.
        std::string env_file_path;
        char *dfcom_ini_path = std::getenv( "DFCOM_INI_PATH" );
        if ( nullptr != dfcom_ini_path )
        {
            env_file_path = dfcom_ini_path;
#if defined(_MSC_VER) || defined(__MINGW32__)
            // Windows akzeptiert auch '/' als Pfadtrenner, somit Gemeinsamkeit mit GNU/Linux herstellen.
            boost::replace_all( env_file_path, "\\", "/" );
#endif
            if ( env_file_path.size() )
            {
                if ( '/' == env_file_path[env_file_path.size() - 1] )
                {
                    env_file_path[env_file_path.size() - 1] = 0;
                }

                // Nur, wenn das Verzeichnis existiert und keine Ini vorliegt, wird eine Standard-Ini erzeugt.
                struct stat lstat;
                if ( stat( env_file_path.c_str(), &lstat ) == 0 && (lstat.st_mode & S_IFMT) == S_IFDIR )
                {
                    ini_file_path = env_file_path + "/DFCom.ini";
                    if ( stat( ini_file_path.c_str(), &lstat ) == -1 || (lstat.st_mode & S_IFMT) != S_IFREG )
                    {
                        if ( create_if_not_exist )
                        {
                            if ( !createIniFile( ini_file_path.c_str(), env_file_path.c_str() ) )
                            {
                                ini_file_path.clear();
                            }
                        }
                        else
                        {
                            ini_file_path.clear();
                        }
                    }
                }
            }
        }
    }

    if ( !ini_file_path.size() )
    {
        // 3. Versuchen die DFCom.ini im App-Data oder Home-Verzeichnis zu finden.
        // Ggf. "create_if_not_exist" eine Standarddatei erzeugen.
        std::string env_file_path;
#if defined(_MSC_VER) || defined(__MINGW32__)
        char *dfcom_ini_path = std::getenv( "LOCALAPPDATA" );
        if ( nullptr != dfcom_ini_path )
        {
            env_file_path = dfcom_ini_path;
        }
        // Windows akzeptiert auch '/' als Pfadtrenner, somit Gemeinsamkeit mit GNU/Linux herstellen.
        boost::replace_all( env_file_path, "\\", "/" );
#else
        char *dfcom_ini_path = std::getenv( "HOME" );
        if ( nullptr != dfcom_ini_path )
        {
            env_file_path = dfcom_ini_path;
        }
#endif
        if ( env_file_path.size() )
        {
            if ( '/' == env_file_path[env_file_path.size() - 1] )
            {
                env_file_path[env_file_path.size() - 1] = 0;
            }

            // Prüfen ob das Verzeichnis existiert.
            struct stat lstat;
            if ( stat( env_file_path.c_str(), &lstat ) == 0 && (lstat.st_mode & S_IFMT) == S_IFDIR )
            {
                // Prüfen ob das Unterverzeichnis existiert.
                env_file_path += "/datafox";
                if ( stat( env_file_path.c_str(), &lstat ) == -1 || (lstat.st_mode & S_IFMT) != S_IFDIR )
                {
                    if ( create_if_not_exist )
                    {
#if defined(_MSC_VER) || defined(__MINGW32__)
                        if ( _mkdir( env_file_path.c_str() ) != 0 )
#else
                        if ( mkdir( env_file_path.c_str(), S_IRWXU | S_IRWXG | S_IROTH | S_IXOTH ) != 0 )
#endif
                        {
                            env_file_path.clear();
                        }
                    }
                    else
                    {
                        env_file_path.clear();
                    }
                }

                if ( env_file_path.size() )
                {
                    ini_file_path = env_file_path;
                    ini_file_path += "/DFCom.ini";
                    if ( stat( ini_file_path.c_str(), &lstat ) == -1 || (lstat.st_mode & S_IFMT) != S_IFREG )
                    {
                        if ( create_if_not_exist )
                        {
                            if ( !createIniFile( ini_file_path.c_str(), env_file_path.c_str() ) )
                            {
                                ini_file_path.clear();
                            }
                        }
                        else
                        {
                            ini_file_path.clear();
                        }
                    }
                }
            }
        }
    }

    return ini_file_path;
}
